/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem B. Boolean Satisfiability                              */
/*                                                                */
/* Original idea         Gennady Korotkevich                      */
/* Problem statement     Pavel Mavrin                             */
/* Test set              Pavel Mavrin                             */
/******************************************************************/
/* Tests                                                          */
/*                                                                */
/* Author                Pavel Mavrin                             */
/******************************************************************/

import java.io.*;
import java.nio.file.*;
import java.util.*;

public class Tests {
    public static final int LETTERS = 52;
    private static int nTests = 0;
    private static final Path root = Paths.get("tests");
    private static int MAX_LEN = 1000;
    private static Set<String> used = new HashSet<>();

    private static final Random random = new Random(7612361223641231123L);
    private static int STEP = 9;

    public static void main(String[] args) throws IOException {
        Files.createDirectories(root);

        test("a");
        test("B|~B");
        test("c|~C");
        test("i|c|p|c");

        test("~Z");
        for (int n = 2; n <= LETTERS; n += (n % STEP != LETTERS % STEP ? 1 : STEP)) {
//            System.out.println(n);
            for (int q = 0; q < 2; q++) {
                boolean[] z = new boolean[LETTERS];
                int[] a = new int[n];
                for (int i = 0; i < n; i++) {
                    int j = random.nextInt(LETTERS);
                    while (z[j]) j = random.nextInt(LETTERS);
                    z[j] = true;
                    a[i] = random.nextBoolean() ? -(j + 1) : (j + 1);
                }
                if (q == 1) {
                    int i = random.nextInt(n);
                    int j = random.nextInt(n);
                    while (i == j) {
                        i = random.nextInt(n);
                        j = random.nextInt(n);
                    }
                    a[i] = -a[j];

                }
                test(a);
            }
        }
        {
            int[] a = new int[MAX_LEN / 2];
            Arrays.fill(a, 1);
            test(a);
            a[a.length - 1] = -1;
            test(a);
        }
        for (int n = 2; n <= LETTERS; n += (n % STEP != LETTERS % STEP ? 1 : STEP)) {
//            System.out.println(n);
            for (int q = 0; q < 2; q++) {
                boolean[] z = new boolean[LETTERS];
                for (int i = 0; i < n; i++) {
                    int j = random.nextInt(LETTERS);
                    while (z[j]) j = random.nextInt(LETTERS);
                    z[j] = true;
                }
                int[] st = new int[LETTERS];
                for (int i = 0; i < st.length; i++) {
                    st[i] = random.nextBoolean() ? -(i + 1) : (i + 1);
                }
                int[] a = new int[MAX_LEN];
                int k = 0;
                int len = 0;
                while (true) {
                    int j = random.nextInt(LETTERS);
                    if (!z[j]) continue;
                    if (len > 0) len++;
                    if (st[j] < 0)
                        len += 2;
                    else
                        len += 1;
                    if (len > MAX_LEN) break;
                    a[k++] = st[j];
                }
                if (q == 1) {
                    int i = random.nextInt(k);
                    int j = random.nextInt(k);
                    while (i == j) {
                        i = random.nextInt(k);
                        j = random.nextInt(k);
                    }
                    a[i] = -a[j];
                }
                test(a);
            }
        }
        {
            int[] a = new int[MAX_LEN];
            int k = 0;
            int len = 0;
            while (true) {
                int j = random.nextInt(LETTERS);
                if (len > 0) len++;
                int x = random.nextBoolean() ? -(j + 1) : (j + 1);
                if (x < 0)
                    len += 2;
                else
                    len += 1;
                if (len > MAX_LEN) break;
                a[k++] = x;
            }
            test(a);
        }
    }

    private static void test(int[] a) {
        int n = 0;
        while (n < a.length && a[n] != 0) n++;
        String[] b = new String[n];
        for (int i = 0; i < n; i++) {
            b[i] = a[i] > 0 ? "" + letter(a[i]) : "~" + letter(-a[i]);
        }
        test(b);
    }

    private static char letter(int i) {
        if (i <= 26) {
            return (char) ('a' + i - 1);
        } else {
            return (char) ('A' + i - 27);
        }
    }


    private static void test(String[] a) {
        String s = String.join("|", a);
        test(s);
    }

    private static void test(String s) {
        if (!used.add(s)) {
            return;
        }
        try (PrintWriter out = new PrintWriter(String.format("%s/%02d", root, ++nTests))) {
            System.out.println("Writing test " + nTests);
            out.println(s);
        } catch (IOException e) {
            throw new AssertionError(e);
        }
    }
}

