/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem H. Hidden Supervisors                                  */
/*                                                                */
/* Original idea         Niyaz Nigmatullin                        */
/* Problem statement     Niyaz Nigmatullin                        */
/* Test set              Niyaz Nigmatullin                        */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Georgiy Korneev                          */
/******************************************************************/

import java.io.*;
import java.util.*;

public class hidden_gk {
    static Scanner in;
    static PrintWriter out;

    static class N {
        int index, empty;
        List<N> children = new ArrayList<>();
        N parent;
        N special;
        List<N> free;


        N(final int index) {
            this.index = index;
        }

        void dfs() {
            for (N child : children) {
                child.dfs();
                empty += child.size();
                if (special == null && child.special == null) {
                    special = child;
                }
            }
        }

        private int size() {
            return empty + (special != null ? 1 : 0);
        }

        void setParent(final N parent) {
            this.parent = parent;
            parent.children.add(this);
        }

        void free(final boolean used, Collection<N> free) {
            for (N c : children) {
                c.free(!used && c == special, free);
            }
            if (!used && special == null) {
                free.add(this);
            }
        }

        @Override
        public String toString() {
            return "" + index;
        }
    }

    static class Solution {
        void run() {
            int n = in.nextInt();
            List<N> nodes = new ArrayList<>();
            for (int i = 0; i < n; i++) {
                final N node = new N(i + 1);
                nodes.add(node);
            }
            for (N node : nodes.subList(1, n)) {
                final int p = in.nextInt();
                if (p != 0) {
                    node.setParent(nodes.get(p - 1));
                }
            }

            Deque<N> free = new ArrayDeque<>();
            Deque<N> candidates = new ArrayDeque<>();
            int size = 0;
            final N root = nodes.get(0);
            for (N node : nodes) {
                if (node.parent == null) {
                    node.dfs();
                    size += node.size();
                    if (node != root) {
                        if (node.special != null) {
                            node.parent = root;
                            node.free(false, free);
                        } else {
                            node.free = new ArrayList<>();
                            node.free(true, node.free);
                            if (node.free.isEmpty()) {
                                candidates.add(node);
                            } else {
                                candidates.push(node);
                            }
                        }
                    } else {
                        node.free(false, free);
                    }
                }
            }

            while (!candidates.isEmpty()) {
                N child = candidates.pop();
                if (!free.isEmpty()) {
                    child.parent = free.pop();
                    free.addAll(child.free);
                    size++;
                } else if (!candidates.isEmpty()) {
                    N parent = candidates.pop();
                    child.parent = parent;
                    parent.parent = root;
                    free.addAll(child.free);
                    free.addAll(parent.free);
                    size++;
                } else {
                    child.parent = root;
                }
            }

            out.println(size);
            for (N node : nodes.subList(1, n)) {
                out.println(node.parent);
            }
        }
    }

    public static void main(String[] args) throws Exception {
        in = new Scanner(new File("hidden.in"));
        out = new PrintWriter("hidden.out");

        new Solution().run();

        in.close();
        out.close();
    }

    static class Scanner {
        final BufferedReader reader;
        StringTokenizer tokenizer = new StringTokenizer("");

        Scanner(File file) throws FileNotFoundException {
            reader = new BufferedReader(new FileReader(file));
        }

        int nextInt() {
            return Integer.parseInt(next());
        }

        String next() {
            while (!tokenizer.hasMoreTokens()) {
                try {
                    tokenizer = new StringTokenizer(reader.readLine());
                } catch (IOException e) {
                    throw new AssertionError();
                }
            }
            return tokenizer.nextToken();
        }

        void close() {
        }
    }
}

