/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem H. Hidden Supervisors                                  */
/*                                                                */
/* Original idea         Niyaz Nigmatullin                        */
/* Problem statement     Niyaz Nigmatullin                        */
/* Test set              Niyaz Nigmatullin                        */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Niyaz Nigmatullin                        */
/******************************************************************/

import java.io.*
import java.util.*

private class SolutionNNKt(val sc: FastScanner) {

    class FastScanner(file: File) {
        var reader = BufferedReader(FileReader(file))
        var tokenizer = StringTokenizer("")

        fun nextInt() = next().toInt()

        private fun next(): String {
            while (!tokenizer.hasMoreTokens()) {
                tokenizer = StringTokenizer(reader.readLine())
            }
            return tokenizer.nextToken()
        }

    }

    val n = sc.nextInt()
    val p = IntArray(n) { if (it == 0) -1 else sc.nextInt() - 1 }
    val deg = with(p) {
        val d = IntArray(n)
        for (i in 1 until n) {
            if (p[i] >= 0) {
                d[p[i]]++
            }
        }
        d
    }
    val edges = with(p) {
        val edges = Array(n) { IntArray(deg[it]) }
        val cnt = IntArray(n)
        for (i in 1 until n) {
            val parent = p[i]
            if (parent >= 0) {
                edges[parent][cnt[parent]++] = i
            }
        }
        edges
    }
    val mSize = IntArray(n)
    val freeSize = IntArray(n)
    val freeVertices = IntArray(n)
    var cntFree = 0

    fun dfs(v: Int): Boolean {
        var can = true
        var have = 1
        var size = 0
        for (to in edges[v]) {
            val got = dfs(to)
            if (got && can) {
                can = false
                have -= 2
                size++
            }
            have += freeSize[to]
            size += mSize[to]
        }
        mSize[v] = size
        freeSize[v] = have
        return can
    }

    fun addFree(v: Int): Boolean {
        var isFree = true
        for (to in edges[v]) {
            if (addFree(to)) {
                if (isFree) {
                    isFree = false
                } else {
                    freeVertices[cntFree++] = to
                }
            }
        }
        return isFree
    }

    fun solve() {
        val answer = p.clone()
        val freeRoots = arrayListOf<Pair<Int, Int>>()
        dfs(0)
        var ans = mSize[0]
        if (addFree(0)) {
            freeVertices[cntFree++] = 0
        }
        for (v in 1 until n) {
            if (p[v] >= 0) continue
            val got = dfs(v)
            ans += mSize[v]
            if (!got) {
                addFree(v)
                answer[v] = 0
            } else {
                freeRoots.add(Pair(freeSize[v], v))
            }
        }
        Collections.sort(freeRoots, compareBy { -it.first })
        for (cur in freeRoots) {
            val v = cur.second
            if (cntFree > 0) {
                ans++
                answer[v] = freeVertices[--cntFree]
                addFree(v)
            } else {
                answer[v] = 0
                if (addFree(v)) {
                    freeVertices[cntFree++] = v
                }
            }
        }
        PrintWriter("hidden.out").use {
            with(it) {
                println(ans)
                println(answer.drop(1).joinToString(" ", transform = { (it + 1).toString() }))
            }
        }
    }
}


fun main(args: Array<String>) {
    SolutionNNKt(SolutionNNKt.FastScanner(File("hidden.in"))).solve()
}

