/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem J. Joker                                               */
/*                                                                */
/* Original idea         Gennady Korotkevich                      */
/* Problem statement     Borys Minaiev                            */
/* Test set              Borys Minaiev                            */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Borys Minaiev                            */
/******************************************************************/

#include <bits/stdc++.h>
#include <chrono>

using namespace std::chrono;
using namespace std;

vector<int> a;
vector<int> pos;
vector<int> value;

#define MAXN 200000
#define inf ((long long) 1e18)

int sumPositivePt[MAXN];
int sumNegativePt[MAXN];
int realPosPt[MAXN];

int vectMul(int a, int b, int c) {
    long long r = ((sumPositivePt[b] - sumPositivePt[a]) * 1LL * (sumNegativePt[c] - sumNegativePt[a]) -
                (sumNegativePt[b] - sumNegativePt[a]) * 1LL * (sumPositivePt[c] - sumPositivePt[a]));
    return (r > 0 ? 1 : (r < 0 ? -1 : 0));
}

class Block {
public:
    int from;   
    int size;
    int sumPositive;
    int sumNegative;

    void setNewValue(int value) {
        if (value > 0) {
            sumPositive = value;
            sumNegative = 0;
        } else {
            sumPositive = 0;
            sumNegative = -value;
        }
        sumNegativePt[from] = sumNegative;
        sumPositivePt[from] = sumPositive;
    }

    int bestPos = -1;

    long long calc(int pos, int prevSumPos, int prevSumNeg, int totalSumPos, int totalSumNeg) {
        return (prevSumPos + sumPositivePt[pos]) * 1LL * totalSumNeg - (prevSumNeg + sumNegativePt[pos]) * 1LL * totalSumPos;
            
    }


    long long findBestAns(int prevSumPos, int prevSumNeg, int totalSumPos, int totalSumNeg) {
        if (size == 1) {
            bestPos = realPosPt[from];
            return calc(from, prevSumPos, prevSumNeg, totalSumPos, totalSumNeg);
        }
        long long res = -inf;
        int l = from, r = from + size;
        while (r - l > 2) {
            int mid = (l + r) >> 1;
            if (calc(mid, prevSumPos, prevSumNeg, totalSumPos, totalSumNeg) < calc(mid + 1, prevSumPos, prevSumNeg, totalSumPos, totalSumNeg)) {
                l = mid;
            } else {
                r = mid + 1;
            }
        }
        for (int i = l; i < r; i++) {
            long long value = calc(i, prevSumPos, prevSumNeg, totalSumPos, totalSumNeg);
            if (value > res) {
                bestPos = realPosPt[i];
                res = value;
            }
        }
        return res;
    }

    long long findBestAnsSlow(int prevSumPos, int prevSumNeg, int totalSumPos, int totalSumNeg) {
        long long res = -inf;
        for (int i = from; i < from + size; i++) {
            long long value = calc(i, prevSumPos, prevSumNeg, totalSumPos, totalSumNeg);
            if (value > res) {
                bestPos = realPosPt[i];
                res = value;
            }
        }
        return res;
    }

    void build(int from1, int to) {
        from = from1;
        size = 0;
        for (int i = from; i < to; i++) {
            if (size > 0 && sumPositivePt[from + size - 1] == sumPositivePt[i]) {
                continue;
            }
            while (size >= 2) {
                if (vectMul(from + size - 2, from + size - 1, i) > 0) {
                    break;
                } else {
                    size--;
                }
            }
            if (size >= 1 && sumNegativePt[from] == sumNegativePt[i]) {
                size--;
            }
            sumPositivePt[from + size] = sumPositivePt[i];
            sumNegativePt[from + size] = sumNegativePt[i];
            realPosPt[from + size] = i;
            size++;
        }

    }


    void create(int from, int to) {
        sumPositive = 0;
        sumNegative = 0;
        for (int i = from; i < to; i++) {
            if (a[i] > 0) {
                sumPositive += a[i];
            } else {
                sumNegative -= a[i];
            }
            sumNegativePt[i] = sumNegative;
            sumPositivePt[i] = sumPositive;

        }
        build(from, to);
    }
};


int findBestAns(vector<Block> &blocks, int blocksSz, int totalSumPos, int totalSumNeg) {
    int curSumPos = 0, curSumNeg = 0;
    long long result = -inf;
    int bestPos = -1;
    for (int i = 0; i < blocksSz; i++) {
        Block &b = blocks[i];
        long long res = b.findBestAns(curSumPos, curSumNeg, totalSumPos, totalSumNeg);
        if (res > result) {
            result = res;
            bestPos = b.bestPos;
        }
        curSumPos += b.sumPositive;
        curSumNeg += b.sumNegative;
    }
    return bestPos;
}

vector<int> solveCorrect() {
    int changes = pos.size();
    vector<int> res(changes + 1);

    int SQRT = 200;
    int n = a.size();
    cerr << "n = " << n << endl;
    vector<bool> toBeChanged(n);

    vector<Block> blocks(2 * SQRT + 3);

    int sumPositive = 0;
    int sumNegative = 0;
    for (int i = 0; i < n; i++) {
        if (a[i] > 0) {
            sumPositive += a[i];
        } else {
            sumNegative -= a[i];
        }
    }
    for (int it = 0; it < changes; it += SQRT) {
        for (int i = 0; i < n; i++) {
            toBeChanged[i] = false;
        }
        for (int i = it; i < it + SQRT && i < changes; i++) {
            toBeChanged[pos[i]] = true;
        }
        int blocksSz = 0;
        int convexSize = 0;
        for (int i = 0; i < n; ) {
            int j = i + 1;
            if (!toBeChanged[i]) {
                while (j != n && !toBeChanged[j]) {
                    j++;
                }
            }
            blocks[blocksSz++].create(i, j);
            i = j;
        }
        if (it == 0) {
            res[0] = findBestAns(blocks, blocksSz, sumPositive, sumNegative);
        }
        for (int i = it; i < it + SQRT && i < changes; i++) {
            int posi = pos[i];
            int nval = value[i];
            if (a[posi] > 0) {
                sumPositive -= a[posi];
            } else {
                sumNegative += a[posi];
            }
            a[posi] = nval;
            if (nval > 0) {
                sumPositive += nval;
            } else {
                sumNegative -= nval;
            }
            // TODO: use binary search?
            for (int j = 0; j < blocksSz; j++) {
                if (blocks[j].from == posi) {
                    blocks[j].setNewValue(nval);
                    break;
                }
            }
            res[i + 1] = findBestAns(blocks, blocksSz, sumPositive, sumNegative);
        }

    }
    return res;
}

int main() {
    long long start = duration_cast<milliseconds>(system_clock::now().time_since_epoch()).count();
    assert(freopen("joker.in", "r", stdin));
    assert(freopen("joker.out", "w", stdout));
    int n, m;
    assert(2 == scanf("%d %d", &n, &m));
    a.resize(n);
    for (int i = 0; i < n; i++) {
        assert(1 == scanf("%d", &a[i]));
    }
    pos.resize(m);
    value.resize(m);
    for (int i = 0; i < m; i++) {
        assert(2 == scanf("%d %d", &pos[i], &value[i]));
        pos[i]--;
    }
    vector<int> res = solveCorrect();
    for (int i = 0; i < res.size(); i++) {
        printf("%d\n", res[i] + 1);
    }
    long long end = duration_cast<milliseconds>(system_clock::now().time_since_epoch()).count();
    cerr << "done in " << (end - start) << " ms" << endl;
}

